package player

import (
	"context"
	jsoniter "github.com/json-iterator/go"
	mhayaTime "github.com/mhaya/extend/time"
	"github.com/mhaya/game/game_cluster/internal/code"
	"github.com/mhaya/game/game_cluster/internal/constant"
	"github.com/mhaya/game/game_cluster/internal/event"
	"github.com/mhaya/game/game_cluster/internal/mdb"
	"github.com/mhaya/game/game_cluster/internal/mdb/models"
	"github.com/mhaya/game/game_cluster/internal/param"
	"github.com/mhaya/game/game_cluster/internal/token"
	"github.com/mhaya/game/game_cluster/nodes/game/module/online"
	clog "github.com/mhaya/logger"
	"github.com/mhaya/net/parser/pomelo"
	"go.mongodb.org/mongo-driver/bson"
	"time"
)

type (
	// actorPlayer 每位登录的玩家对应一个子actor
	actorPlayer struct {
		pomelo.ActorBase
		isOnline bool // 玩家是否在线
		playerId int64
		uid      int64
		index    int
		dirty    bool
		Player   *models.Player
	}
)

func (p *actorPlayer) OnInit() {
	clog.Debugf("[actorPlayer] path = %s init!", p.PathString())

	// 注册 session关闭的remote函数(网关触发连接断开后，会调用RPC发送该消息)
	p.Remote().Register("sessionClose", p.sessionClose)

	p.Remote().Register("login", p.login) // 注册 查看角色

	p.Remote().Register("start", p.start) // 主页

	p.Remote().Register("guide", p.guide) // 主页关闭信息

	p.Remote().Register("roll", p.roll) // roll

	p.Remote().Register("lottery", p.lottery) // 转盘

	p.Remote().Register("signIn", p.signIn) // 签到

	p.Remote().Register("rank", p.rank) // 排行榜

	p.Remote().Register("invite", p.invite) // 邀请

	p.Remote().Register("share", p.share) // 分享

	p.Remote().Register("claim", p.claim) // 领取奖励

	p.Remote().Register("jump", p.jump)               // 成就1相关的跳转服务
	p.Remote().Register("savex", p.savex)             // 更新推特
	p.Remote().Register("savetonwall", p.savetonwall) // 更新钱包

	p.Timer().Add(10*time.Second, p.refreshRoll)
	p.Timer().Add(1*time.Second, p.SetRewardToDB)
	p.Timer().Add(1*time.Minute, p.setRedis)
	p.Timer().Add(1*time.Second, p.setDb)
	p.Timer().Add(30*time.Second, p.setInviteReward)

	p.Timer().Add(1*time.Second, p.addRoll)
}

func (p *actorPlayer) addRoll() {
	if p.isOnline {
		if p.Player.Item[models.ItemRoll].Num < 10 {
			p.Player.Item[models.ItemRoll].Num += 100
		}
		if p.Player.Item[models.ItemDrawsNumber].Num < 10 {
			p.Player.Item[models.ItemDrawsNumber].Num += 100
		}
		p.dirty = true
	}
}

func (p *actorPlayer) OnStop() {
	clog.Debugf("[actorPlayer] path = %s exit!", p.PathString())
}

func (p *actorPlayer) setInviteReward() {
	if p.isOnline {
		p.Player.SetInviteReward()
	}
}

func (p *actorPlayer) setRedis() {
	if p.isOnline {
		var playerBase = param.PlayerBase{
			UserName:      p.Player.UserName,
			NickName:      p.Player.NickName,
			UserNameMaybe: p.Player.UserNameMaybe,
			IsLeader:      p.Player.IsLeader,
			Avatar:        p.Player.Avatar,
			OpenId:        p.Player.OpenId,
		}
		d, _ := jsoniter.Marshal(&playerBase)
		mdb.RDB.Set(context.Background(), models.GetPlayBaseKey(p.Player.UserName), d, 30*24*time.Hour)
	}
}

func (p *actorPlayer) SetRewardToDB() {
	if p.Player != nil {
		for k, r := range p.Player.PlayerReward {
			_, err := mdb.MDB.Collection(constant.CNamePlayerReward).InsertOne(context.Background(), r)
			if err != nil {
				clog.Errorf("[actorPlayer] set invite to db error: %v", err)
			}
			delete(p.Player.PlayerReward, k)
		}
	}
}

func (p *actorPlayer) setDb() {
	if p.isOnline && p.dirty {
		ret, err := mdb.MDB.Collection(constant.CNamePlayer).UpdateOne(context.Background(), bson.M{"userName": p.Player.UserName}, bson.M{"$set": p.Player})
		if err != nil {
			clog.Debugf("[actorPlayer] setDb ret=%v, err = %s exit!", ret, err)
		}
		p.dirty = false
	}
}

func (p *actorPlayer) refreshRoll() {
	if p.isOnline {
		if p.Player.RefreshRoll() {
			p.dirty = true
		}
	}
}

// sessionClose 接收角色session关闭处理
func (p *actorPlayer) sessionClose() {
	online.UnBindPlayer(p.uid)
	p.isOnline = false
	p.Exit()

	logoutEvent := event.NewPlayerLogout(p.ActorID(), p.playerId)
	p.PostEvent(&logoutEvent)
}
func (p *actorPlayer) Init(req *token.Token) {
	clog.Debugf("[actorPlayer] init token = %s", req)
	var player models.Player
	mdb.MDB.Collection(constant.CNamePlayer).FindOne(context.Background(), bson.M{"userName": req.PlayerID}).Decode(&player)
	if player.OpenId != "" {
		p.Player = &player
	} else {
		user := models.NewPlayer(req)
		mdb.MDB.Collection(constant.CNamePlayer).InsertOne(context.Background(), user)
		p.Player = user
	}
	if len(p.Player.PlayerReward) == 0 {
		p.Player.PlayerReward = make(map[int]*models.PlayerRewardBase)
	}
}

func (p *actorPlayer) InitBase(uid string) int32 {
	clog.Debugf("[actorPlayer] InitBase uid = %s", uid)
	var player models.Player
	mdb.MDB.Collection(constant.CNamePlayer).FindOne(context.Background(), bson.M{"userName": uid}).Decode(&player)
	if player.OpenId != "" {
		p.Player = &player
		if len(p.Player.PlayerReward) == 0 {
			p.Player.PlayerReward = make(map[int]*models.PlayerRewardBase)
		}
		return code.OK
	} else {
		return code.Error
	}
}

func (p *actorPlayer) login(req *token.Token) (*models.Player, int32) {
	if !p.isOnline {
		p.Init(req)
	}
	p.isOnline = true
	p.dirty = true

	p.Player.Init()

	p.Player.NickName = req.Nickname
	if !mhayaTime.CreateFromTimestamp(p.Player.LoginTime).IsToday() {
		p.Player.Successions += 1
		if mhayaTime.CreateFromTimestamp(p.Player.LoginTime).DiffInDays(mhayaTime.Now()) > 1 {
			p.Player.MaxSuccessions = 1
		} else {
			p.Player.MaxSuccessions += 1
		}
	}
	p.Player.LoginTime = mhayaTime.Now().Unix()
	//刷新roll
	p.Player.RefreshRoll()
	p.Player.InitDaily()
	p.Player.InitWeekly()
	p.Player.InitAchieveTask()

	return p.Player, code.OK
}
